@@ -23,9 +23,14 @@ try:
     import threading
 except ImportError:
     import dummy_threading as threading
-import numpy as np
-from collections import OrderedDict
+try:
+    from contextlib import nullcontext
+except ImportError:
+    from contextlib import ExitStack as nullcontext  # Py 3.6.
 from math import radians, cos, sin
+
+import numpy as np
+
 from matplotlib import cbook, rcParams, __version__
 from matplotlib.backend_bases import (
     _Backend, FigureCanvasBase, FigureManagerBase, RendererBase)
@@ -38,7 +43,6 @@ from matplotlib.transforms import Bbox, BboxBase
 from matplotlib import colors as mcolors
 
 from matplotlib.backends._backend_agg import RendererAgg as _RendererAgg
-from matplotlib import _png
 
 from matplotlib.backend_bases import _has_pil
 
@@ -47,6 +51,7 @@ if _has_pil:
 
 backend_version = 'v2.2'
 
+
 def get_hinting_flag():
     mapping = {
         True: LOAD_FORCE_AUTOHINT,
@@ -118,10 +123,8 @@ class RendererAgg(RendererBase):
         return np.array(region), extents
 
     def draw_path(self, gc, path, transform, rgbFace=None):
-        """
-        Draw the path
-        """
-        nmax = rcParams['agg.path.chunksize'] # here at least for testing
+        # docstring inherited
+        nmax = rcParams['agg.path.chunksize']  # here at least for testing
         npts = path.vertices.shape[0]
 
         if (nmax > 100 and npts > nmax and path.should_simplify and
@@ -160,14 +163,13 @@ class RendererAgg(RendererBase):
 
         xd = descent * sin(radians(angle))
         yd = descent * cos(radians(angle))
-        x = np.round(x + ox + xd)
-        y = np.round(y - oy + yd)
+        x = round(x + ox + xd)
+        y = round(y - oy + yd)
         self._renderer.draw_text_image(font_image, x, y + 1, angle, gc)
 
     def draw_text(self, gc, x, y, s, prop, angle, ismath=False, mtext=None):
-        """
-        Render the text
-        """
+        # docstring inherited
+
         if ismath:
             return self.draw_mathtext(gc, x, y, s, prop, angle)
 
@@ -176,33 +178,26 @@ class RendererAgg(RendererBase):
 
         if font is None:
             return None
-        if len(s) == 1 and ord(s) > 127:
-            font.load_char(ord(s), flags=flags)
-        else:
-            # We pass '0' for angle here, since it will be rotated (in raster
-            # space) in the following call to draw_text_image).
-            font.set_text(s, 0, flags=flags)
+        # We pass '0' for angle here, since it will be rotated (in raster
+        # space) in the following call to draw_text_image).
+        font.set_text(s, 0, flags=flags)
         font.draw_glyphs_to_bitmap(antialiased=rcParams['text.antialiased'])
         d = font.get_descent() / 64.0
         # The descent needs to be adjusted for the angle.
         xo, yo = font.get_bitmap_offset()
         xo /= 64.0
         yo /= 64.0
-        xd = -d * sin(radians(angle))
+        xd = d * sin(radians(angle))
         yd = d * cos(radians(angle))
-
-        self._renderer.draw_text_image(
-            font, np.round(x - xd + xo), np.round(y + yd + yo) + 1, angle, gc)
+        x = round(x + xo + xd)
+        y = round(y + yo + yd)
+        self._renderer.draw_text_image(font, x, y + 1, angle, gc)
 
     def get_text_width_height_descent(self, s, prop, ismath):
-        """
-        Get the width, height, and descent (offset from the bottom
-        to the baseline), in display coords, of the string *s* with
-        :class:`~matplotlib.font_manager.FontProperties` *prop*
-        """
+        # docstring inherited
+
         if ismath in ["TeX", "TeX!"]:
             # todo: handle props
-            size = prop.get_size_in_points()
             texmanager = self.get_texmanager()
             fontsize = prop.get_size_in_points()
             w, h, d = texmanager.get_text_width_height_descent(
@@ -225,6 +220,7 @@ class RendererAgg(RendererBase):
         return w, h, d
 
     def draw_tex(self, gc, x, y, s, prop, angle, ismath='TeX!', mtext=None):
+        # docstring inherited
         # todo, handle props, angle, origins
         size = prop.get_size_in_points()
 
@@ -236,13 +232,12 @@ class RendererAgg(RendererBase):
         w, h, d = self.get_text_width_height_descent(s, prop, ismath)
         xd = d * sin(radians(angle))
         yd = d * cos(radians(angle))
-        x = np.round(x + xd)
-        y = np.round(y + yd)
-
+        x = round(x + xd)
+        y = round(y + yd)
         self._renderer.draw_text_image(Z, x, y, angle, gc)
 
     def get_canvas_width_height(self):
-        'return the canvas width and height in display coords'
+        # docstring inherited
         return self.width, self.height
 
     def _get_agg_font(self, prop):
@@ -259,41 +254,38 @@ class RendererAgg(RendererBase):
         return font
 
     def points_to_pixels(self, points):
-        """
-        convert point measures to pixes using dpi and the pixels per
-        inch of the display
-        """
+        # docstring inherited
         return points * self.dpi / 72
 
-    def tostring_rgb(self):
-        return self._renderer.tostring_rgb()
+    def buffer_rgba(self):
+        return memoryview(self._renderer)
 
     def tostring_argb(self):
-        return self._renderer.tostring_argb()
+        return np.asarray(self._renderer).take([3, 0, 1, 2], axis=2).tobytes()
 
-    def buffer_rgba(self):
-        return self._renderer.buffer_rgba()
+    def tostring_rgb(self):
+        return np.asarray(self._renderer).take([0, 1, 2], axis=2).tobytes()
 
     def clear(self):
         self._renderer.clear()
 
     def option_image_nocomposite(self):
+        # docstring inherited
+
         # It is generally faster to composite each image directly to
         # the Figure, and there's no file size benefit to compositing
         # with the Agg backend
         return True
 
     def option_scale_image(self):
-        """
-        agg backend doesn't support arbitrary scaling of image.
-        """
+        # docstring inherited
         return False
 
     def restore_region(self, region, bbox=None, xy=None):
         """
         Restore the saved region. If bbox (instance of BboxBase, or
         its extents) is given, only the region specified by the bbox
-        will be restored. *xy* (a tuple of two floasts) optionally
+        will be restored. *xy* (a pair of floats) optionally
         specifies the new position (the LLC of the original region,
         not the LLC of the bbox) where the region will be restored.
 
@@ -359,7 +351,7 @@ class RendererAgg(RendererBase):
         self._update_methods()
 
         if w > 0 and h > 0:
-            img = np.fromstring(buffer, np.uint8)
+            img = np.frombuffer(buffer, np.uint8)
             img, ox, oy = post_processing(img.reshape((h, w, 4)) / 255.,
                                           self.dpi)
             gc = self.new_gc()
@@ -394,26 +386,21 @@ class FigureCanvasAgg(FigureCanvasBase):
         Draw the figure using the renderer.
         """
         self.renderer = self.get_renderer(cleared=True)
-        # acquire a lock on the shared font cache
-        RendererAgg.lock.acquire()
-
-        toolbar = self.toolbar
-        try:
+        # Acquire a lock on the shared font cache.
+        with RendererAgg.lock, \
+             (self.toolbar._wait_cursor_for_draw_cm() if self.toolbar
+              else nullcontext()):
             self.figure.draw(self.renderer)
             # A GUI class may be need to update a window using this draw, so
             # don't forget to call the superclass.
             super().draw()
-        finally:
-            RendererAgg.lock.release()
 
     def get_renderer(self, cleared=False):
         l, b, w, h = self.figure.bbox.bounds
         key = w, h, self.figure.dpi
-        try: self._lastKey, self.renderer
-        except AttributeError: need_new_renderer = True
-        else:  need_new_renderer = (self._lastKey != key)
-
-        if need_new_renderer:
+        reuse_renderer = (hasattr(self, "renderer")
+                          and getattr(self, "_lastKey", None) == key)
+        if not reuse_renderer:
             self.renderer = RendererAgg(w, h, self.figure.dpi)
             self._lastKey = key
         elif cleared:
@@ -421,7 +408,7 @@ class FigureCanvasAgg(FigureCanvasBase):
         return self.renderer
 
     def tostring_rgb(self):
-        '''Get the image as an RGB byte string.
+        """Get the image as an RGB byte string.
 
         `draw` must be called at least once before this function will work and
         to update the renderer for any subsequent changes to the Figure.
@@ -429,11 +416,11 @@ class FigureCanvasAgg(FigureCanvasBase):
         Returns
         -------
         bytes
-        '''
+        """
         return self.renderer.tostring_rgb()
 
     def tostring_argb(self):
-        '''Get the image as an ARGB byte string
+        """Get the image as an ARGB byte string.
 
         `draw` must be called at least once before this function will work and
         to update the renderer for any subsequent changes to the Figure.
@@ -441,31 +428,32 @@ class FigureCanvasAgg(FigureCanvasBase):
         Returns
         -------
         bytes
-
-        '''
+        """
         return self.renderer.tostring_argb()
 
     def buffer_rgba(self):
-        '''Get the image as an RGBA byte string.
+        """Get the image as a memoryview to the renderer's buffer.
 
         `draw` must be called at least once before this function will work and
         to update the renderer for any subsequent changes to the Figure.
 
         Returns
         -------
-        bytes
-        '''
+        memoryview
+        """
         return self.renderer.buffer_rgba()
 
     def print_raw(self, filename_or_obj, *args, **kwargs):
         FigureCanvasAgg.draw(self)
         renderer = self.get_renderer()
-        with cbook._setattr_cm(renderer, dpi=self.figure.dpi), \
-                cbook.open_file_cm(filename_or_obj, "wb") as fh:
-            fh.write(renderer._renderer.buffer_rgba())
+        with cbook.open_file_cm(filename_or_obj, "wb") as fh:
+            fh.write(renderer.buffer_rgba())
+
     print_rgba = print_raw
 
-    def print_png(self, filename_or_obj, *args, **kwargs):
+    def print_png(self, filename_or_obj, *args,
+                  metadata=None, pil_kwargs=None,
+                  **kwargs):
         """
         Write the figure to a PNG file.
 
@@ -506,32 +494,63 @@ class FigureCanvasAgg(FigureCanvasBase):
             .. _PNG specification: \
                 https://www.w3.org/TR/2003/REC-PNG-20031110/#11keywords
 
+        pil_kwargs : dict, optional
+            If set to a non-None value, use Pillow to save the figure instead
+            of Matplotlib's builtin PNG support, and pass these keyword
+            arguments to `PIL.Image.save`.
+
+            If the 'pnginfo' key is present, it completely overrides
+            *metadata*, including the default 'Software' key.
         """
+        from matplotlib import _png
+
+        if metadata is None:
+            metadata = {}
+        default_metadata = {
+            "Software":
+                f"matplotlib version{__version__}, http://matplotlib.org/",
+        }
+
         FigureCanvasAgg.draw(self)
-        renderer = self.get_renderer()
+        if pil_kwargs is not None:
+            from PIL import Image
+            from PIL.PngImagePlugin import PngInfo
+            # Only use the metadata kwarg if pnginfo is not set, because the
+            # semantics of duplicate keys in pnginfo is unclear.
+            if "pnginfo" in pil_kwargs:
+                if metadata:
+                    cbook._warn_external("'metadata' is overridden by the "
+                                         "'pnginfo' entry in 'pil_kwargs'.")
+            else:
+                pnginfo = PngInfo()
+                for k, v in {**default_metadata, **metadata}.items():
+                    pnginfo.add_text(k, v)
+                pil_kwargs["pnginfo"] = pnginfo
+            pil_kwargs.setdefault("dpi", (self.figure.dpi, self.figure.dpi))
+            (Image.fromarray(np.asarray(self.buffer_rgba()))
+             .save(filename_or_obj, format="png", **pil_kwargs))
 
-        version_str = (
-            'matplotlib version ' + __version__ + ', http://matplotlib.org/')
-        metadata = OrderedDict({'Software': version_str})
-        user_metadata = kwargs.pop("metadata", None)
-        if user_metadata is not None:
-            metadata.update(user_metadata)
-
-        with cbook._setattr_cm(renderer, dpi=self.figure.dpi), \
-                cbook.open_file_cm(filename_or_obj, "wb") as fh:
-            _png.write_png(renderer._renderer, fh,
-                            self.figure.dpi, metadata=metadata)
+        else:
+            renderer = self.get_renderer()
+            with cbook.open_file_cm(filename_or_obj, "wb") as fh:
+                _png.write_png(renderer._renderer, fh, self.figure.dpi,
+                               metadata={**default_metadata, **metadata})
 
     def print_to_buffer(self):
         FigureCanvasAgg.draw(self)
         renderer = self.get_renderer()
-        with cbook._setattr_cm(renderer, dpi=self.figure.dpi):
-            return (renderer._renderer.buffer_rgba(),
-                    (int(renderer.width), int(renderer.height)))
+        return (bytes(renderer.buffer_rgba()),
+                (int(renderer.width), int(renderer.height)))
 
     if _has_pil:
-        # add JPEG support
-        def print_jpg(self, filename_or_obj, *args, dryrun=False, **kwargs):
+
+        # Note that these methods should typically be called via savefig() and
+        # print_figure(), and the latter ensures that `self.figure.dpi` already
+        # matches the dpi kwarg (if any).
+
+        @cbook._delete_parameter("3.2", "dryrun")
+        def print_jpg(self, filename_or_obj, *args, dryrun=False,
+                      pil_kwargs=None, **kwargs):
             """
             Write the figure to a JPEG file.
 
@@ -556,36 +575,44 @@ class FigureCanvasAgg(FigureCanvasBase):
             progressive : bool
                 If present, indicates that this image
                 should be stored as a progressive JPEG file.
+
+            pil_kwargs : dict, optional
+                Additional keyword arguments that are passed to
+                `PIL.Image.save` when saving the figure.  These take precedence
+                over *quality*, *optimize* and *progressive*.
             """
-            buf, size = self.print_to_buffer()
+            FigureCanvasAgg.draw(self)
             if dryrun:
                 return
-            # The image is "pasted" onto a white background image to safely
-            # handle any transparency
-            image = Image.frombuffer('RGBA', size, buf, 'raw', 'RGBA', 0, 1)
-            rgba = mcolors.to_rgba(rcParams['savefig.facecolor'])
-            color = tuple([int(x * 255) for x in rgba[:3]])
-            background = Image.new('RGB', size, color)
+            # The image is pasted onto a white background image to handle
+            # transparency.
+            image = Image.fromarray(np.asarray(self.buffer_rgba()))
+            background = Image.new('RGB', image.size, "white")
             background.paste(image, image)
-            options = {k: kwargs[k]
-                       for k in ['quality', 'optimize', 'progressive', 'dpi']
-                       if k in kwargs}
-            options.setdefault('quality', rcParams['savefig.jpeg_quality'])
-            if 'dpi' in options:
-                # Set the same dpi in both x and y directions
-                options['dpi'] = (options['dpi'], options['dpi'])
+            if pil_kwargs is None:
+                pil_kwargs = {}
+            for k in ["quality", "optimize", "progressive"]:
+                if k in kwargs:
+                    pil_kwargs.setdefault(k, kwargs[k])
+            pil_kwargs.setdefault("quality", rcParams["savefig.jpeg_quality"])
+            pil_kwargs.setdefault("dpi", (self.figure.dpi, self.figure.dpi))
+            return background.save(
+                filename_or_obj, format='jpeg', **pil_kwargs)
 
-            return background.save(filename_or_obj, format='jpeg', **options)
         print_jpeg = print_jpg
 
-        # add TIFF support
-        def print_tif(self, filename_or_obj, *args, dryrun=False, **kwargs):
-            buf, size = self.print_to_buffer()
+        @cbook._delete_parameter("3.2", "dryrun")
+        def print_tif(self, filename_or_obj, *args, dryrun=False,
+                      pil_kwargs=None, **kwargs):
+            FigureCanvasAgg.draw(self)
             if dryrun:
                 return
-            image = Image.frombuffer('RGBA', size, buf, 'raw', 'RGBA', 0, 1)
-            dpi = (self.figure.dpi, self.figure.dpi)
-            return image.save(filename_or_obj, format='tiff', dpi=dpi)
+            if pil_kwargs is None:
+                pil_kwargs = {}
+            pil_kwargs.setdefault("dpi", (self.figure.dpi, self.figure.dpi))
+            return (Image.fromarray(np.asarray(self.buffer_rgba()))
+                    .save(filename_or_obj, format='tiff', **pil_kwargs))
+
         print_tiff = print_tif
 
 
