@@ -1,6 +1,6 @@
 """
-The OffsetBox is a simple container artist. The child artist are meant
-to be drawn at a relative position to its parent.  The [VH]Packer,
+The `.OffsetBox` is a simple container artist. Its child artists are
+meant to be drawn at a relative position to OffsetBox.  The [VH]Packer,
 DrawingArea and TextArea are derived from the OffsetBox.
 
 The [VH]Packer automatically adjust the relative positions of their
@@ -49,12 +49,48 @@ def _get_packed_offsets(wd_list, total, sep, mode="fixed"):
     *mode*. xdescent is analogous to the usual descent, but along the
     x-direction. xdescent values are currently ignored.
 
-    *wd_list* : list of (width, xdescent) of boxes to be packed.
-    *sep* : spacing between boxes
-    *total* : Intended total length. None if not used.
-    *mode* : packing mode. 'fixed', 'expand', or 'equal'.
-    """
+    For simplicity of the description, the terminology used here assumes a
+    horizontal layout, but the function works equally for a vertical layout.
 
+    There are three packing modes:
+
+    - 'fixed': The elements are packed tight to the left with a spacing of
+      *sep* in between. If *total* is *None* the returned total will be the
+      right edge of the last box. A non-*None* total will be passed unchecked
+      to the output. In particular this means that right edge of the last
+      box may be further to the right than the returned total.
+
+    - 'expand': Distribute the boxes with equal spacing so that the left edge
+      of the first box is at 0, and the right edge of the last box is at
+      *total*. The parameter *sep* is ignored in this mode. A total of *None*
+      is accepted and considered equal to 1. The total is returned unchanged
+      (except for the conversion *None* to 1). If the total is smaller than
+      the sum of the widths, the laid out boxes will overlap.
+
+    - 'equal': If *total* is given, the total space is divided in N equal
+      ranges and each box is left-aligned within its subspace.
+      Otherwise (*total* is *None*), *sep* must be provided and each box is
+      left-aligned in its subspace of width ``(max(widths) + sep)``. The
+      total width is then calculated to be ``N * (max(widths) + sep)``.
+
+    Parameters
+    ----------
+    wd_list : list of (float, float)
+        (width, xdescent) of boxes to be packed.
+    total : float or None
+        Intended total length. *None* if not used.
+    sep : float
+        Spacing between boxes.
+    mode : {'fixed', 'expand', 'equal'}
+        The packing mode.
+
+    Returns
+    -------
+    total : float
+        The total width needed to accommodate the laid out boxes.
+    offsets : array of float
+        The left offsets of the boxes.
+    """
     w_list, d_list = zip(*wd_list)
     # d_list is currently not used.
 
@@ -81,6 +117,9 @@ def _get_packed_offsets(wd_list, total, sep, mode="fixed"):
     elif mode == "equal":
         maxh = max(w_list)
         if total is None:
+            if sep is None:
+                raise ValueError("total and sep cannot both be None when "
+                                 "using layout mode 'equal'.")
             total = (maxh + sep) * len(w_list)
         else:
             sep = total / len(w_list) - maxh
@@ -147,9 +186,11 @@ class OffsetBox(martist.Artist):
 
     def set_figure(self, fig):
         """
-        Set the figure
+        Set the `.Figure` for the `.OffsetBox` and all its children.
 
-        accepts a class:`~matplotlib.figure.Figure` instance
+        Parameters
+        ----------
+        fig : `~matplotlib.figure.Figure`
         """
         martist.Artist.set_figure(self, fig)
         for c in self.get_children():
@@ -164,6 +205,32 @@ class OffsetBox(martist.Artist):
                 c.axes = ax
 
     def contains(self, mouseevent):
+        """
+        Delegate the mouse event contains-check to the children.
+
+        As a container, the `.OffsetBox` does not respond itself to
+        mouseevents.
+
+        Parameters
+        ----------
+        mouseevent : `matplotlib.backend_bases.MouseEvent`
+
+        Returns
+        -------
+        contains : bool
+            Whether any values are within the radius.
+        details : dict
+            An artist-specific dictionary of details of the event context,
+            such as which points are contained in the pick radius. See the
+            individual Artist subclasses for details.
+
+        See Also
+        --------
+        .Artist.contains
+        """
+        inside, info = self._default_contains(mouseevent)
+        if inside is not None:
+            return inside, info
         for c in self.get_children():
             a, b = c.contains(mouseevent)
             if a:
@@ -177,8 +244,10 @@ class OffsetBox(martist.Artist):
         Parameters
         ----------
         xy : (float, float) or callable
-            The (x,y) coordinates of the offset in display units.
-            A callable must have the signature::
+            The (x, y) coordinates of the offset in display units. These can
+            either be given explicitly as a tuple (x, y), or by providing a
+            function that converts the extent into the offset. This function
+            must have the signature::
 
                 def offset(width, height, xdescent, ydescent, renderer) \
 -> (float, float)
@@ -188,9 +257,18 @@ class OffsetBox(martist.Artist):
 
     def get_offset(self, width, height, xdescent, ydescent, renderer):
         """
-        Get the offset
+        Return the offset as a tuple (x, y).
+
+        The extent parameters have to be provided to handle the case where the
+        offset is dynamically determined by a callable (see
+        `~.OffsetBox.set_offset`).
+
+        Parameters
+        ----------
+        width, height, xdescent, ydescent
+            Extent parameters.
+        renderer : `.RendererBase` subclass
 
-        accepts extent of the box
         """
         return (self._offset(width, height, xdescent, ydescent, renderer)
                 if callable(self._offset)
@@ -198,48 +276,60 @@ class OffsetBox(martist.Artist):
 
     def set_width(self, width):
         """
-        Set the width
+        Set the width of the box.
 
-        accepts float
+        Parameters
+        ----------
+        width : float
         """
         self.width = width
         self.stale = True
 
     def set_height(self, height):
         """
-        Set the height
+        Set the height of the box.
 
-        accepts float
+        Parameters
+        ----------
+        height : float
         """
         self.height = height
         self.stale = True
 
     def get_visible_children(self):
-        """
-        Return a list of visible artists it contains.
-        """
+        r"""Return a list of the visible child `.Artist`\s."""
         return [c for c in self._children if c.get_visible()]
 
     def get_children(self):
-        """
-        Return a list of artists it contains.
-        """
+        r"""Return a list of the child `.Artist`\s."""
         return self._children
 
     def get_extent_offsets(self, renderer):
-        raise Exception("")
+        """
+        Update offset of the children and return the extent of the box.
+
+        Parameters
+        ----------
+        renderer : `.RendererBase` subclass
+
+        Returns
+        -------
+        width
+        height
+        xdescent
+        ydescent
+        list of (xoffset, yoffset) pairs
+        """
+        raise NotImplementedError(
+            "get_extent_offsets must be overridden in derived classes.")
 
     def get_extent(self, renderer):
-        """
-        Return with, height, xdescent, ydescent of box
-        """
+        """Return a tuple ``width, height, xdescent, ydescent`` of the box."""
         w, h, xd, yd, offsets = self.get_extent_offsets(renderer)
         return w, h, xd, yd
 
     def get_window_extent(self, renderer):
-        '''
-        get the bounding box in display space.
-        '''
+        """Return the bounding box (`.Bbox`) in display space."""
         w, h, xd, yd, offsets = self.get_extent_offsets(renderer)
         px, py = self.get_offset(w, h, xd, yd, renderer)
         return mtransforms.Bbox.from_bounds(px - xd, py - yd, w, h)
@@ -249,7 +339,6 @@ class OffsetBox(martist.Artist):
         Update the location of children if necessary and draw them
         to the given *renderer*.
         """
-
         width, height, xdescent, ydescent, offsets = self.get_extent_offsets(
                                                         renderer)
 
@@ -271,23 +360,29 @@ class PackerBase(OffsetBox):
         Parameters
         ----------
         pad : float, optional
-            Boundary pad.
+            The boundary padding in points.
 
         sep : float, optional
-            Spacing between items.
+            The spacing between items in points.
 
-        width : float, optional
+        width, height : float, optional
+            Width and height of the container box in pixels, calculated if
+            *None*.
 
-        height : float, optional
-           Width and height of the container box, calculated if
-           `None`.
+        align : {'top', 'bottom', 'left', 'right', 'center', 'baseline'}
+            Alignment of boxes.
 
-        align : str, optional
-            Alignment of boxes. Can be one of ``top``, ``bottom``,
-            ``left``, ``right``, ``center`` and ``baseline``
+        mode : {'fixed', 'expand', 'equal'}
+            The packing mode.
 
-        mode : str, optional
-            Packing mode.
+            - 'fixed' packs the given `.Artists` tight with *sep* spacing.
+            - 'expand' uses the maximal available space to distribute the
+              artists with equal spacing in between.
+            - 'equal': Each artist an equal fraction of the available space
+              and is left-aligned (or top-aligned) therein.
+
+        children : list of `.Artist`
+            The artists to pack.
 
         Notes
         -----
@@ -319,23 +414,29 @@ class VPacker(PackerBase):
         Parameters
         ----------
         pad : float, optional
-            Boundary pad.
+            The boundary padding in points.
 
         sep : float, optional
-            Spacing between items.
+            The spacing between items in points.
 
-        width : float, optional
+        width, height : float, optional
+            Width and height of the container box in pixels, calculated if
+            *None*.
 
-        height : float, optional
-
-            width and height of the container box, calculated if
-            `None`.
-
-        align : str, optional
+        align : {'top', 'bottom', 'left', 'right', 'center', 'baseline'}
             Alignment of boxes.
 
-        mode : str, optional
-            Packing mode.
+        mode : {'fixed', 'expand', 'equal'}
+            The packing mode.
+
+            - 'fixed' packs the given `.Artists` tight with *sep* spacing.
+            - 'expand' uses the maximal available space to distribute the
+              artists with equal spacing in between.
+            - 'equal': Each artist an equal fraction of the available space
+              and is left-aligned (or top-aligned) therein.
+
+        children : list of `.Artist`
+            The artists to pack.
 
         Notes
         -----
@@ -346,10 +447,7 @@ class VPacker(PackerBase):
         super().__init__(pad, sep, width, height, align, mode, children)
 
     def get_extent_offsets(self, renderer):
-        """
-        update offset of childrens and return the extents of the box
-        """
-
+        # docstring inherited
         dpicor = renderer.points_to_pixels(1.)
         pad = self.pad * dpicor
         sep = self.sep * dpicor
@@ -395,22 +493,29 @@ class HPacker(PackerBase):
         Parameters
         ----------
         pad : float, optional
-            Boundary pad.
+            The boundary padding in points.
 
         sep : float, optional
-            Spacing between items.
+            The spacing between items in points.
 
-        width : float, optional
+        width, height : float, optional
+            Width and height of the container box in pixels, calculated if
+            *None*.
 
-        height : float, optional
-           Width and height of the container box, calculated if
-           `None`.
+        align : {'top', 'bottom', 'left', 'right', 'center', 'baseline'}
+            Alignment of boxes.
 
-        align : str
-           Alignment of boxes.
+        mode : {'fixed', 'expand', 'equal'}
+            The packing mode.
 
-        mode : str
-           Packing mode.
+            - 'fixed' packs the given `.Artists` tight with *sep* spacing.
+            - 'expand' uses the maximal available space to distribute the
+              artists with equal spacing in between.
+            - 'equal': Each artist an equal fraction of the available space
+              and is left-aligned (or top-aligned) therein.
+
+        children : list of `.Artist`
+            The artists to pack.
 
         Notes
         -----
@@ -421,9 +526,7 @@ class HPacker(PackerBase):
         super().__init__(pad, sep, width, height, align, mode, children)
 
     def get_extent_offsets(self, renderer):
-        """
-        update offset of children and return the extents of the box
-        """
+        # docstring inherited
         dpicor = renderer.points_to_pixels(1.)
         pad = self.pad * dpicor
         sep = self.sep * dpicor
@@ -458,20 +561,30 @@ class HPacker(PackerBase):
 
         return (width + 2 * pad, height + 2 * pad,
                 xdescent + pad, ydescent + pad,
-               list(zip(xoffsets, yoffsets)))
+                list(zip(xoffsets, yoffsets)))
 
 
 class PaddedBox(OffsetBox):
+    """
+    A container to add a padding around an `.Artist`.
+
+    The `.PaddedBox` contains a `.FancyBboxPatch` that is used to visualize
+    it when rendering.
+    """
     def __init__(self, child, pad=None, draw_frame=False, patch_attrs=None):
         """
-        *pad* : boundary pad
-
-        .. note::
-          *pad* need to given in points and will be
-          scale with the renderer dpi, while *width* and *height*
-          need to be in pixels.
+        Parameters
+        ----------
+        child : `~matplotlib.artist.Artist`
+            The contained `.Artist`.
+        pad : float
+            The padding in points. This will be scaled with the renderer dpi.
+            In contrast *width* and *hight* are in *pixel* and thus not scaled.
+        draw_frame : bool
+            Whether to draw the contained `.FancyBboxPatch`.
+        patch_attrs : dict or None
+            Additional parameters passed to the contained `.FancyBboxPatch`.
         """
-
         super().__init__()
 
         self.pad = pad
@@ -492,25 +605,18 @@ class PaddedBox(OffsetBox):
         self._drawFrame = draw_frame
 
     def get_extent_offsets(self, renderer):
-        """
-        update offset of childrens and return the extents of the box
-        """
-
+        # docstring inherited.
         dpicor = renderer.points_to_pixels(1.)
         pad = self.pad * dpicor
-
         w, h, xd, yd = self._children[0].get_extent(renderer)
-
-        return w + 2 * pad, h + 2 * pad, \
-               xd + pad, yd + pad, \
-               [(0, 0)]
+        return (w + 2 * pad, h + 2 * pad, xd + pad, yd + pad,
+                [(0, 0)])
 
     def draw(self, renderer):
         """
         Update the location of children if necessary and draw them
         to the given *renderer*.
         """
-
         width, height, xdescent, ydescent, offsets = self.get_extent_offsets(
                                                         renderer)
 
@@ -559,19 +665,13 @@ class DrawingArea(OffsetBox):
         *xdescent*, *ydescent* : descent of the box in x- and y-direction.
         *clip* : Whether to clip the children
         """
-
         super().__init__()
-
         self.width = width
         self.height = height
         self.xdescent = xdescent
         self.ydescent = ydescent
         self._clip_children = clip
-
         self.offset_transform = mtransforms.Affine2D()
-        self.offset_transform.clear()
-        self.offset_transform.translate(0, 0)
-
         self.dpi_transform = mtransforms.Affine2D()
 
     @property
@@ -589,8 +689,7 @@ class DrawingArea(OffsetBox):
 
     def get_transform(self):
         """
-        Return the :class:`~matplotlib.transforms.Transform` applied
-        to the children
+        Return the `~matplotlib.transforms.Transform` applied to the children.
         """
         return self.dpi_transform + self.offset_transform
 
@@ -598,7 +697,6 @@ class DrawingArea(OffsetBox):
         """
         set_transform is ignored.
         """
-        pass
 
     def set_offset(self, xy):
         """
@@ -607,10 +705,9 @@ class DrawingArea(OffsetBox):
         Parameters
         ----------
         xy : (float, float)
-            The (x,y) coordinates of the offset in display units.
+            The (x, y) coordinates of the offset in display units.
         """
         self._offset = xy
-
         self.offset_transform.clear()
         self.offset_transform.translate(xy[0], xy[1])
         self.stale = True
@@ -636,8 +733,8 @@ class DrawingArea(OffsetBox):
         """
 
         dpi_cor = renderer.points_to_pixels(1.)
-        return self.width * dpi_cor, self.height * dpi_cor, \
-               self.xdescent * dpi_cor, self.ydescent * dpi_cor
+        return (self.width * dpi_cor, self.height * dpi_cor,
+                self.xdescent * dpi_cor, self.ydescent * dpi_cor)
 
     def add_artist(self, a):
         'Add any :class:`~matplotlib.artist.Artist` to the container box'
@@ -657,7 +754,7 @@ class DrawingArea(OffsetBox):
 
         dpi_cor = renderer.points_to_pixels(1.)
         self.dpi_transform.clear()
-        self.dpi_transform.scale(dpi_cor, dpi_cor)
+        self.dpi_transform.scale(dpi_cor)
 
         # At this point the DrawingArea has a transform
         # to the display space so the path created is
@@ -679,7 +776,7 @@ class DrawingArea(OffsetBox):
 class TextArea(OffsetBox):
     """
     The TextArea is contains a single Text instance. The text is
-    placed at (0,0) with baseline+left alignment. The width and height
+    placed at (0, 0) with baseline+left alignment. The width and height
     of the TextArea instance is the width and height of the its child
     text.
     """
@@ -707,23 +804,14 @@ class TextArea(OffsetBox):
         """
         if textprops is None:
             textprops = {}
-
-        if "va" not in textprops:
-            textprops["va"] = "baseline"
-
+        textprops.setdefault("va", "baseline")
         self._text = mtext.Text(0, 0, s, **textprops)
-
         OffsetBox.__init__(self)
-
         self._children = [self._text]
-
         self.offset_transform = mtransforms.Affine2D()
-        self.offset_transform.clear()
-        self.offset_transform.translate(0, 0)
         self._baseline_transform = mtransforms.Affine2D()
         self._text.set_transform(self.offset_transform +
                                  self._baseline_transform)
-
         self._multilinebaseline = multilinebaseline
         self._minimumdescent = minimumdescent
 
@@ -772,7 +860,6 @@ class TextArea(OffsetBox):
         """
         set_transform is ignored.
         """
-        pass
 
     def set_offset(self, xy):
         """
@@ -781,10 +868,9 @@ class TextArea(OffsetBox):
         Parameters
         ----------
         xy : (float, float)
-            The (x,y) coordinates of the offset in display units.
+            The (x, y) coordinates of the offset in display units.
         """
         self._offset = xy
-
         self.offset_transform.clear()
         self.offset_transform.translate(xy[0], xy[1])
         self.stale = True
@@ -849,7 +935,7 @@ class AuxTransformBox(OffsetBox):
     transformed with the aux_transform first then will be
     offseted. The absolute coordinate of the aux_transform is meaning
     as it will be automatically adjust so that the left-lower corner
-    of the bounding box of children will be set to (0,0) before the
+    of the bounding box of children will be set to (0, 0) before the
     offset transform.
 
     It is similar to drawing area, except that the extent of the box
@@ -860,16 +946,10 @@ class AuxTransformBox(OffsetBox):
     def __init__(self, aux_transform):
         self.aux_transform = aux_transform
         OffsetBox.__init__(self)
-
         self.offset_transform = mtransforms.Affine2D()
-        self.offset_transform.clear()
-        self.offset_transform.translate(0, 0)
-
-        # ref_offset_transform is used to make the offset_transform is
-        # always reference to the lower-left corner of the bbox of its
-        # children.
+        # ref_offset_transform makes offset_transform always relative to the
+        # lower-left corner of the bbox of its children.
         self.ref_offset_transform = mtransforms.Affine2D()
-        self.ref_offset_transform.clear()
 
     def add_artist(self, a):
         'Add any :class:`~matplotlib.artist.Artist` to the container box'
@@ -882,15 +962,14 @@ class AuxTransformBox(OffsetBox):
         Return the :class:`~matplotlib.transforms.Transform` applied
         to the children
         """
-        return self.aux_transform + \
-               self.ref_offset_transform + \
-               self.offset_transform
+        return (self.aux_transform
+                + self.ref_offset_transform
+                + self.offset_transform)
 
     def set_transform(self, t):
         """
         set_transform is ignored.
         """
-        pass
 
     def set_offset(self, xy):
         """
@@ -899,10 +978,9 @@ class AuxTransformBox(OffsetBox):
         Parameters
         ----------
         xy : (float, float)
-            The (x,y) coordinates of the offset in display units.
+            The (x, y) coordinates of the offset in display units.
         """
         self._offset = xy
-
         self.offset_transform.clear()
         self.offset_transform.translate(xy[0], xy[1])
         self.stale = True
@@ -922,22 +1000,17 @@ class AuxTransformBox(OffsetBox):
         return mtransforms.Bbox.from_bounds(ox - xd, oy - yd, w, h)
 
     def get_extent(self, renderer):
-
         # clear the offset transforms
-        _off = self.offset_transform.to_values()  # to be restored later
+        _off = self.offset_transform.get_matrix()  # to be restored later
         self.ref_offset_transform.clear()
         self.offset_transform.clear()
-
         # calculate the extent
         bboxes = [c.get_window_extent(renderer) for c in self._children]
         ub = mtransforms.Bbox.union(bboxes)
-
         # adjust ref_offset_transform
         self.ref_offset_transform.translate(-ub.x0, -ub.y0)
-
         # restor offset transform
-        mtx = self.offset_transform.matrix_from_values(*_off)
-        self.offset_transform.set_matrix(mtx)
+        self.offset_transform.set_matrix(_off)
 
         return ub.width, ub.height, 0., 0.
 
@@ -1019,12 +1092,7 @@ class AnchoredOffsetbox(OffsetBox):
         self.set_child(child)
 
         if isinstance(loc, str):
-            try:
-                loc = self.codes[loc]
-            except KeyError:
-                raise ValueError('Unrecognized location "%s". Valid '
-                                 'locations are\n\t%s\n'
-                                 % (loc, '\n\t'.join(self.codes)))
+            loc = cbook._check_getitem(self.codes, loc=loc)
 
         self.loc = loc
         self.borderpad = borderpad
@@ -1210,7 +1278,7 @@ class AnchoredText(AnchoredOffsetbox):
         """
         Parameters
         ----------
-        s : string
+        s : str
             Text.
 
         loc : str
@@ -1240,7 +1308,7 @@ class AnchoredText(AnchoredOffsetbox):
             cbook.warn_deprecated(
                 "3.1", message="Mixing horizontalalignment or "
                 "verticalalignment with AnchoredText is not supported, "
-                "deprecated since %(version)s, and will raise an exception "
+                "deprecated since %(since)s, and will raise an exception "
                 "%(removal)s.")
 
         self.txt = TextArea(s, textprops=prop, minimumdescent=False)
@@ -1309,7 +1377,7 @@ class OffsetImage(OffsetBox):
 #         Parameters
 #         ----------
 #         xy : (float, float)
-#             The (x,y) coordinates of the offset in display units.
+#             The (x, y) coordinates of the offset in display units.
 #         """
 #         self._offset = xy
 
@@ -1455,10 +1523,13 @@ class AnnotationBbox(martist.Artist, _AnnotationBase):
         self.boxcoords = coords
         self.stale = True
 
-    def contains(self, event):
-        t, tinfo = self.offsetbox.contains(event)
+    def contains(self, mouseevent):
+        inside, info = self._default_contains(mouseevent)
+        if inside is not None:
+            return inside, info
+        t, tinfo = self.offsetbox.contains(mouseevent)
         #if self.arrow_patch is not None:
-        #    a,ainfo=self.arrow_patch.contains(event)
+        #    a, ainfo=self.arrow_patch.contains(event)
         #    t = t or a
 
         # self.arrow_patch is currently not checked as this can be a line - JJ
@@ -1594,7 +1665,7 @@ class AnnotationBbox(martist.Artist, _AnnotationBase):
         self.stale = False
 
 
-class DraggableBase(object):
+class DraggableBase:
     """
     Helper base class for a draggable artist (legend, offsetbox).
 
@@ -1654,7 +1725,7 @@ class DraggableBase(object):
             self.update_offset(dx, dy)
             self.canvas.restore_region(self.background)
             self.ref_artist.draw(self.ref_artist.figure._cachedRenderer)
-            self.canvas.blit(self.ref_artist.figure.bbox)
+            self.canvas.blit()
 
     def on_pick(self, evt):
         if self._check_still_parented() and evt.artist == self.ref_artist:
@@ -1669,7 +1740,7 @@ class DraggableBase(object):
                 self.background = self.canvas.copy_from_bbox(
                                     self.ref_artist.figure.bbox)
                 self.ref_artist.draw(self.ref_artist.figure._cachedRenderer)
-                self.canvas.blit(self.ref_artist.figure.bbox)
+                self.canvas.blit()
                 self._c1 = self.canvas.mpl_connect('motion_notify_event',
                                                    self.on_motion_blit)
             else:
